<?php

namespace App\Services;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use App\ProductCategory;
use App\Product;
use App\ProductItem;
use App\ProductVariation;

class Import
{
    /**
     * This function consumes the import request and initiates the import.
     *
     * @param String $typ
     * @return Success / Exception
     * @author @mattsidjohn
     **/
    public function run($types)
    {
        try {
            // Consume the import request into its requested types.
            $types = explode(':', $types);

            // Iterate through the types and run the requested import.
            foreach ($types as $type) {
                // Extract contents of export file.
                $contents = Storage::get('imports/' .$type . '.txt');

                // Transpose into a collectable object.
                $collection = collect((object) json_decode($contents));

                // Send the collection and type to the receiver of the import
                $this->receive($type, $collection);
            }
        } catch (Exception $e) {
            return $e;
        }
    }

    /**
     * This function is the receiver for the import
     *
     * @param String $type, Object $collection
     * @return Success / Exception
     * @author @mattsidjohn
     **/
    private function receive($type, $collection)
    {
        // Set transaction listener to monitor db transactions
        DB::beginTransaction();

        // Initiate try and catch malformed data insertions resulting in an error
        try {
            // Route Import to the correct type.
            switch ($type) {
                case 'productCategories':
                    $this->importProductCategories($collection);
                    break;
                case 'products':
                    $this->importProducts($collection);
                    break;
                case 'items':
                    $this->importItems($collection);
                    break;
                case 'variations':
                    $this->importVariations($collection);
                    break;
                default:
                    return 'No Import Type Available';
            }
            // Commit transactions to the DB
            DB::commit();
        } catch (Exception $e) {
            // Roll back transactions and return Exception
            DB::rollback();
            return $e;
        }
    }

    /**
     * This function imports products from a given collection
     *
     * @param Object $categories
     * @author @mattsidjohn
     **/
    private function importProductCategories($categories)
    {
        // Loop through all categories and look to see if exists, and update or create.
        foreach ($categories as $category) {
            ProductCategory::updateOrCreate(
                ['ee_category_id' => $category->ee_category_id],
                collect($category)->toArray()
            );
        }
    }

    /**
     * This function imports products from a given collection
     *
     * @param Object $products
     * @author @mattsidjohn
     **/
    private function importProducts($products)
    {
        // Loop through all products and look to see if exists, and update or create.
        foreach ($products as $product) {
            $insert = collect($product);
            $insert->forget(['colors', 'categories']);

            $model = Product::updateOrCreate(
                ['ee_product_id' => $product->ee_product_id],
                collect($insert)->toArray()
            );

            // Store converted Ccolors to a serialized array of objects
            $model->colors = $this->serializeColors($product->colors);

            // Relate product categories to the product.
            $model->categories()->sync($this->relatableCategoryIds($product->categories));

            // Save the models relationships and new data.
            $model->save();
        }
    }

    /**
     * This function imports items from a given collection
     *
     * @param Object $items
     * @author @mattsidjohn
     **/
    private function importItems($items)
    {
        // Loop through all products and look to see if exists, and update or create.
        foreach ($items as $item) {
            // Find the app product_id
            $product = Product::where('ee_product_id', $item->ee_product_id)->first();

            // Add app product id to the item object
            $item->product_id = $product->id;

            // Create / Update Item
            ProductItem::updateOrCreate(
                [
                    'ee_product_id' => $product->ee_product_id,
                    'item_code' => $item->item_code
                ],
                collect($item)->toArray()
            );
        }
    }

    /**
     * This function imports variations from a given collection
     *
     * @param Object $variations
     * @return N/A
     * @author @mattsidjohn
     **/
    private function importVariations($variations)
    {

        // Loop through all products and look to see if exists, and update or create.
        foreach ($variations as $variation) {
            // Find the app product_id
            $item = ProductItem::where('item_code', $variation->item_relationship)->first();

            // If item exists set the product_id relationship
            !empty($item) and $variation->product_id = $item->product_id;

            // Create / Update Item
            $model = ProductVariation::updateOrCreate(
                ['item_number' => $variation->item_number],
                collect($variation)->toArray()
            );
        }
    }

    /**
     * This function returns a serialized array of color objects
     *
     * @param String $colors
     * @return Serialized object
     * @author @mattsidjohn
     **/
    private function serializeColors($colors)
    {
        // Iterate through colors to extract the details of each color into an array of objects.
        foreach (explode('|', $colors) as $color) {
            $details = explode(':', $color);
            $array[] = (object) [
                'hex' => $details[0],
                'hex2' => $details[2],
                'name' => $details[1]
            ];
        }

        return serialize($array);
    }
    
    /**
     * This function returns an array of categories id's to associate the products to
     * if the category doesn't exist it will create a new one and return it's id.
     *
     * @param String $categories
     * @return array of relatable category id's for the product
     * @author @mattsidjohn
     **/
    private function relatableCategoryIds($categories)
    {
        // Return array of relatable ids
        return ProductCategory::whereIn(
            'name',
            explode(',', $categories)
        )->pluck('id');
    }
}
